'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _extends = Object.assign || function (target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i]; for (var key in source) { if (Object.prototype.hasOwnProperty.call(source, key)) { target[key] = source[key]; } } } return target; };

var _createClass = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }();

var _object = require('object.values');

var _object2 = _interopRequireDefault(_object);

var _genId = require('./genId');

var _genId2 = _interopRequireDefault(_genId);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

/*
 * Manager that calculates grid and item styles using absolute attributes
 *
 * This manager is used for compatibility with browsers like ie11, which does
 * not support CSS display grid. It uses absolute positioning to position the
 * items in the grid, and it means that
 *
 * Possible improvements:
 *  - The recalculate could use a timer mechanism, so that it doesn't start
 *    recalculating immediately when mounting items. With a setTimeout we
 *    could delay recalculate a bit while everything gets mounted
 *  - Some updates doesn't require full recalculations. For example, when a new
 *    item gets added we only need to recalculate style for that item, and
 *    we only need to recalculate grid style if this item goes beyond current
 *    grid's height
 */
var CSSAbsoluteManager = function () {
  function CSSAbsoluteManager() {
    _classCallCheck(this, CSSAbsoluteManager);

    this.items = {};
  }

  _createClass(CSSAbsoluteManager, [{
    key: 'setContainerWidth',
    value: function setContainerWidth(containerWidth) {
      this.containerWidth = containerWidth;

      this.recalculate();
    }
  }, {
    key: 'subscribeGrid',
    value: function subscribeGrid(callback) {
      var _this = this;

      this.grid = { callback: callback };

      return {
        update: function update(config) {
          _this.grid.config = config;

          _this.recalculate();
        },
        unsubscribe: function unsubscribe() {
          return _this.grid = null;
        }
      };
    }
  }, {
    key: 'subscribeItem',
    value: function subscribeItem(callback) {
      var _this2 = this;

      var id = (0, _genId2.default)();

      this.items[id] = { callback: callback, config: {} };

      return {
        update: function update(config) {
          _this2.items[id] = _extends({}, _this2.items[id], { config: config });

          _this2.recalculate();
        },
        unsubscribe: function unsubscribe() {
          delete _this2.items[id];

          _this2.recalculate();
        }
      };
    }
  }, {
    key: 'recalculate',
    value: function recalculate() {
      var containerWidth = this.containerWidth,
          grid = this.grid,
          items = this.items;


      if (!containerWidth || !grid || !grid.config) {
        return;
      }

      var _grid$config = grid.config,
          columnCount = _grid$config.columnCount,
          gutterSize = _grid$config.gutterSize;


      var containerWidthInVW = containerWidth * 100 / document.body.offsetWidth;
      var totalGutterSize = gutterSize * (columnCount - 1);
      var columnWidth = (containerWidthInVW - totalGutterSize) / columnCount;

      var maxRowSpan = 0;

      (0, _object2.default)(items).forEach(function (item) {
        var callback = item.callback,
            _item$config = item.config,
            _item$config$origin = _item$config.origin,
            columnOrigin = _item$config$origin.column,
            rowOrigin = _item$config$origin.row,
            _item$config$span = _item$config.span,
            columnSpan = _item$config$span.column,
            rowSpan = _item$config$span.row;


        var width = columnWidth * columnSpan + (columnSpan - 1) * gutterSize;
        var height = columnWidth * rowSpan + (rowSpan - 1) * gutterSize;
        var left = (columnOrigin - 1) * (columnWidth + gutterSize);
        var top = (rowOrigin - 1) * (columnWidth + gutterSize);

        callback({
          position: 'absolute',
          width: width + 'vw',
          height: height + 'vw',
          left: left + 'vw',
          top: top + 'vw'
        });

        var itemRowSpan = rowOrigin + rowSpan - 1;
        maxRowSpan = maxRowSpan > itemRowSpan ? maxRowSpan : itemRowSpan;
      });

      var gridHeight = columnWidth * maxRowSpan + gutterSize * (maxRowSpan - 1);

      grid.callback({
        position: 'relative',
        overflow: 'hidden',
        height: (gridHeight > 0 ? gridHeight : 0) + 'vw'
      });
    }
  }]);

  return CSSAbsoluteManager;
}();

exports.default = CSSAbsoluteManager;
//# sourceMappingURL=index.js.map