'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _createClass = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }();

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

/*
 * Manager that calculates grid and item styles using CSS display grid attributes
 *
 * Given how display grid works, the styles for grid and item are not dependent
 * on each other, meaning that we can immediatly calculate the item style,
 * and when there's a grid configuration of containerWidth change the only
 * thing we need to recalculate is the grid style itself, so it should be
 * the prefered manager whenever possible.
 */
var CSSGridManager = function () {
  function CSSGridManager() {
    _classCallCheck(this, CSSGridManager);
  }

  _createClass(CSSGridManager, [{
    key: 'setContainerWidth',
    value: function setContainerWidth(containerWidth) {
      this.containerWidth = containerWidth;

      this.recalculate();
    }
  }, {
    key: 'subscribeGrid',
    value: function subscribeGrid(callback) {
      var _this = this;

      this.grid = { callback: callback };

      return {
        update: function update(config) {
          _this.grid.config = config;

          _this.recalculate();
        },
        unsubscribe: function unsubscribe() {
          _this.grid = null;
        }
      };
    }
  }, {
    key: 'subscribeItem',
    value: function subscribeItem(callback) {
      var subscribed = true;

      return {
        update: function update(_ref) {
          var origin = _ref.origin,
              span = _ref.span;

          if (subscribed) {
            callback({
              gridColumn: origin.column + ' / span ' + span.column,
              gridRow: origin.row + ' / span ' + span.row
            });
          }
        },
        unsubscribe: function unsubscribe() {
          return subscribed = false;
        }
      };
    }
  }, {
    key: 'recalculate',
    value: function recalculate() {
      var containerWidth = this.containerWidth,
          grid = this.grid;


      if (!containerWidth || !grid || !grid.config) {
        return;
      }

      var _grid$config = grid.config,
          columnCount = _grid$config.columnCount,
          gutterSize = _grid$config.gutterSize;


      var containerWidthInVW = containerWidth * 100 / document.body.offsetWidth;
      var totalGutterSizeInVW = gutterSize * (columnCount - 1);
      var rowHeightInVW = (containerWidthInVW - totalGutterSizeInVW) / columnCount;

      grid.callback({
        display: 'grid',
        gridGap: gutterSize + 'vw',
        gridTemplateColumns: 'repeat(' + columnCount + ', 1fr)',
        gridAutoRows: rowHeightInVW + 'vw'
      });
    }
  }]);

  return CSSGridManager;
}();

exports.default = CSSGridManager;
//# sourceMappingURL=index.js.map